import sys
import random
import argparse

def generate_test_values(noise_bits, number, p_bits):
    sys.set_int_max_str_digits(10000000)

    p = random.randint(2**(p_bits-1), 2**p_bits)
    while p % 2 == 0:
        p = random.randint(2**(p_bits-1), 2**p_bits)

    max_noise = (1 << noise_bits) - 1  # 2^noise_bits - 1

    a = [str(p * random.randint(1, 2) + random.randint(0, max_noise)) for _ in range(number)]

    return noise_bits, a, p


def generate_test_file(noise_bits, number, p_bits, filename):
    noise_bits, a, p = generate_test_values(noise_bits, number, p_bits)

    with open(filename, 'w') as f:
        f.write(f"{noise_bits}\n")
        for b in a:
            f.write(f"{b}\n")
        f.write(f"// True p: {p}\n")

    return p

def print_test_values(noise_bits, number, p_bits):

    _, a, p = generate_test_values(noise_bits, number, p_bits)

    print(noise_bits)

    for b in a:
        print(b)

    print(f"// True p: {p}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Generate test input for AGCD computation')
    parser.add_argument('--noise-bits', type=int, default=5, help='Number of noise bits (default: 5)')
    parser.add_argument('--p-bits', type=int, default=16, help='Number of key bits (default: 16)')
    parser.add_argument('--number', type=int, default=20, help='Number of numbers to generate (default: 20)')

    args = parser.parse_args()
    print_test_values(args.noise_bits, args.number, args.p_bits)
